searchData={"items":[{"type":"module","title":"PropertyTable","doc":"In-memory key-value store with subscriptions\n\nPropertyTable makes it easy to set up a key-value store where users can subscribe to changes based on patterns. PropertyTable refers to keys as\nproperties. Properties have values and are timestamped as to when they\nreceived that value. Subscriptions make this library feel similar to Publish-Subscribe. Events, though, are only for changes to properties.\n\nPropertyTable is useful when you want to expose a decent amount of state and\nlet consumers pick and choose what parts interest them.\n\nPropertyTable consumers express their interest in properties using \"patterns\". A pattern could be as simple as the property of interest or it could contain\nwildcards. This allows one to create hierarchical key-value stores, map-based\nstores, or just simple key-value stores with notifications.\n\nPropertyTable is optionally persistent to disk. Keys and values are backed by ETS.","ref":"PropertyTable.html"},{"type":"function","title":"PropertyTable.child_spec/1","doc":"Returns a specification to start a property_table under a supervisor.\nSee `Supervisor`.","ref":"PropertyTable.html#child_spec/1"},{"type":"function","title":"PropertyTable.delete/2","doc":"Delete the specified property","ref":"PropertyTable.html#delete/2"},{"type":"function","title":"PropertyTable.delete_matches/2","doc":"Delete all properties that match a pattern","ref":"PropertyTable.html#delete_matches/2"},{"type":"function","title":"PropertyTable.fetch_with_timestamp/2","doc":"Fetch a property with the time that it was set\n\nTimestamps come from `System.monotonic_time()`","ref":"PropertyTable.html#fetch_with_timestamp/2"},{"type":"function","title":"PropertyTable.flush_to_disk/1","doc":"Write any changes to disk\n\nIf persistence is enabled for this property table, save the current state to\ndisk immediately. The table is already written every `:persist_interval`, but\nthis is avoid waiting after important changes.","ref":"PropertyTable.html#flush_to_disk/1"},{"type":"function","title":"PropertyTable.get/3","doc":"Get the current value of a property","ref":"PropertyTable.html#get/3"},{"type":"function","title":"PropertyTable.get_all/1","doc":"Get all properties\n\nThis function might return a really long list so it's mainly intended for\ndebug or convenience when you know that the table only contains a few\nproperties.","ref":"PropertyTable.html#get_all/1"},{"type":"function","title":"PropertyTable.get_snapshots/1","doc":"Return available snapshot IDs\n\nThis scans the `snapshots` directory and returns a list of tuples containing\nsnapshot IDs and their full name.","ref":"PropertyTable.html#get_snapshots/1"},{"type":"function","title":"PropertyTable.match/2","doc":"Get a list of all properties matching the specified property pattern","ref":"PropertyTable.html#match/2"},{"type":"function","title":"PropertyTable.put/3","doc":"Update a property and notify listeners","ref":"PropertyTable.html#put/3"},{"type":"function","title":"PropertyTable.put_many/2","doc":"Update many properties\n\nThis is similar to calling `put/3` several times in a row, but atomically. It is\nalso slightly more efficient when updating more than one property.","ref":"PropertyTable.html#put_many/2"},{"type":"function","title":"PropertyTable.restore_snapshot/2","doc":"Restart a previously saved snapshot\n\nIf persistence is enabled for this property table, restore the current state\nof the PropertyTable to that of a past named snapshot","ref":"PropertyTable.html#restore_snapshot/2"},{"type":"function","title":"PropertyTable.snapshot/1","doc":"Take a snapshot of the property table\n\nIf persistence is enabled for this property table, save the current state\nand copy a snapshot of it into the `/snapshots` sub-directory of the set\ndata directory.\n\nThis returns an ID for the snapshot that can be passed `restore_snapshot/2`.","ref":"PropertyTable.html#snapshot/1"},{"type":"function","title":"PropertyTable.start_link/1","doc":"Start a PropertyTable's supervision tree\n\nTo create a PropertyTable for your application or library, add the following\n`child_spec` to one of your supervision trees:\n\n```elixir\n{PropertyTable, name: MyTableName}\n```\n\nThe `:name` option is required. All calls to `PropertyTable` will need to\nknow it and the process will be registered under than name so be sure it's\nunique.\n\nOptions for properties and events:\n\n* `:properties` - a list of `{property, value}` tuples to initially populate\n  the `PropertyTable`\n* `:matcher` - set the format for how properties and how they should be\n  matched for triggering events. See `PropertyTable.Matcher`.\n* `:tuple_events` - set to `true` for change events to be in the old tuple\n  format. This is not recommended for new code and hopefully will be removed\n  in the future. Prefer `:event_transformer`\n* `:event_transformer` - set to a function that takes an `Event.t()` and returns\n  the event that should be sent to subscribers.\n\nOptions for persisting properties:\n\n* `:persist_data_path` - set to a directory where PropertyTable will\n  persist the contents of the table to disk, snapshots will also be stored here.\n* `:persist_interval` - if set PropertyTable will persist the contents of\n  tables to disk in intervals of the provided value (in milliseconds) automatically.\n* `:persist_max_snapshots` - Maximum number of manual snapshots to keep on disk before they\n  are replaced - (oldest snapshots are replaced first.) Defaults to 25.\n* `:persist_compression` - `0..9` range to compress the terms when written to disk, see `:erlang.term_to_binary/2`. Defaults to 6.\n\n> #### Important {: .info}\n>\n> Setting `:persist_data_path` enables persistence. On initialization, if\n> PropertyTable is able to load a snapshot, the data in the snapshot is used\n> instead of the `:properties` option.","ref":"PropertyTable.html#start_link/1"},{"type":"function","title":"PropertyTable.subscribe/2","doc":"Subscribe to receive property change events\n\nSubscriptions may be for individual properties or contain wildcards as\nallowed by the `PropertyTable.Matcher` that's in use. See\n`PropertyTable.Matcher.StringPath` for the default.\n\n> #### Important {: .warning}\n>\n> A common pattern is to subscribe to a property and then to get its initial\n> value. The order matters. Subscription must be done first to prevent a race\n> condition. For example, if you get a property's value and _then_ subscribe,\n> it's possible for the value to change before the subscription takes effect.","ref":"PropertyTable.html#subscribe/2"},{"type":"function","title":"PropertyTable.unsubscribe/2","doc":"Stop subscribing to a property","ref":"PropertyTable.html#unsubscribe/2"},{"type":"type","title":"PropertyTable.options/0","doc":"PropertyTable configuration options\n\nSee `start_link/1` for usage.","ref":"PropertyTable.html#t:options/0"},{"type":"type","title":"PropertyTable.pattern/0","doc":"A match pattern\n\nJust like properties, these can be anything but they have to be compatible\nwith the `PropertyTable.Matcher` implementation.\n\nThe default is that patterns are string lists with the addition of wildcards\nlike `:_`.","ref":"PropertyTable.html#t:pattern/0"},{"type":"type","title":"PropertyTable.property/0","doc":"A property\n\nProperties can be anything, but in order to be useful, they need to be\ncompatible with the `PropertyTable.Matcher` implementation.\n\nIn is common for this to be a string list (`[String.t()]`) since the default\n`PropertyTable.Matcher` works with those.","ref":"PropertyTable.html#t:property/0"},{"type":"type","title":"PropertyTable.property_value/0","doc":"A property/value tuple","ref":"PropertyTable.html#t:property_value/0"},{"type":"type","title":"PropertyTable.table_id/0","doc":"A table_id identifies a group of properties","ref":"PropertyTable.html#t:table_id/0"},{"type":"type","title":"PropertyTable.value/0","doc":"A properties value\n\nThese can be whatever makes sense to the PropertyTable user. The only\nconstraint is that if you're using PropertyTable's persistence feature, it\nneeds to be possible to save and restore them. This means that pids and\nreferences, for example, can't be used.","ref":"PropertyTable.html#t:value/0"},{"type":"module","title":"PropertyTable.Event","doc":"Struct sent to subscribers on property changes\n\n* `:table` - the table generating this event\n* `:property` - which property changed\n* `:value` - the new value\n* `:timestamp` - the timestamp (`System.monotonic_time/0`) when the changed\n  happened\n* `:previous_value` - the previous value (`nil` if this property is new)\n* `:previous_timestamp` - the timestamp when the property changed to\n  `:previous_value`. Use this to calculate how long the property was the\n  previous value.","ref":"PropertyTable.Event.html"},{"type":"function","title":"PropertyTable.Event.to_tuple/1","doc":"Convert event to the old tuple event format\n\nThis is only used for backwards compatibility. At some point, it hopefully\nwill be removed.","ref":"PropertyTable.Event.html#to_tuple/1"},{"type":"type","title":"PropertyTable.Event.t/0","doc":"","ref":"PropertyTable.Event.html#t:t/0"},{"type":"behaviour","title":"PropertyTable.Matcher","doc":"Behaviour for customizing the Matcher logic for filtering and dispatching events","ref":"PropertyTable.Matcher.html"},{"type":"callback","title":"PropertyTable.Matcher.check_pattern/1","doc":"Check whether a pattern is valid\n\nReturns `:ok` on success or `{:error, error}` where `error` is an `Exception` struct with\ninformation about the issue.","ref":"PropertyTable.Matcher.html#c:check_pattern/1"},{"type":"callback","title":"PropertyTable.Matcher.check_property/1","doc":"Check whether a property is valid\n\nReturns `:ok` on success or `{:error, error}` where `error` is an `Exception` struct with\ninformation about the issue.","ref":"PropertyTable.Matcher.html#c:check_property/1"},{"type":"callback","title":"PropertyTable.Matcher.matches?/2","doc":"Returns true if the pattern matches the specified property","ref":"PropertyTable.Matcher.html#c:matches?/2"},{"type":"module","title":"PropertyTable.Matcher.StringPath","doc":"Match logic using keys organized as hierarchical lists\n\nProperty keys that are lists look like `[\"first\", \"second\", \"third\"]`.\nThese are intended to create a hierarchical organization of keys. Matching\npatterns involves checking whether the pattern is at the beginning of the\nkey. This makes it possible to get notified on every property change where\nthe key begins with `[\"first\", \"second\"]`. This is a really common use case\nwhen using hierarchically organized keys.\n\nTwo special atoms can be used:\n\n* `:_` - match anything at this part of the list\n* `:$` - match the end of the list","ref":"PropertyTable.Matcher.StringPath.html"},{"type":"function","title":"PropertyTable.Matcher.StringPath.check_pattern/1","doc":"Check whether a pattern is valid\n\nReturns `:ok` on success or `{:error, error}` where `error` is an `Exception` struct with\ninformation about the issue.","ref":"PropertyTable.Matcher.StringPath.html#check_pattern/1"},{"type":"function","title":"PropertyTable.Matcher.StringPath.check_property/1","doc":"Check whether a property is valid\n\nReturns `:ok` on success or `{:error, error}` where `error` is an `Exception` struct with\ninformation about the issue.","ref":"PropertyTable.Matcher.StringPath.html#check_property/1"},{"type":"function","title":"PropertyTable.Matcher.StringPath.matches?/2","doc":"Returns true if the pattern matches the specified property","ref":"PropertyTable.Matcher.StringPath.html#matches?/2"},{"type":"extras","title":"PropertyTable","doc":"# PropertyTable\n\n[![CircleCI](https://circleci.com/gh/nerves-project/property_table.svg?style=svg)](https://circleci.com/gh/nerves-project/property_table)\n[![Hex version](https://img.shields.io/hexpm/v/property_table.svg \"Hex version\")](https://hex.pm/packages/property_table)\n[![Coverage Status](https://coveralls.io/repos/github/nerves-project/property_table/badge.svg)](https://coveralls.io/github/nerves-project/property_table)\n\n<!-- MODULEDOC -->\nIn-memory key-value store with subscriptions\n\nPropertyTable makes it easy to set up a key-value store where users can subscribe to changes based on patterns. PropertyTable refers to keys as\nproperties. Properties have values and are timestamped as to when they\nreceived that value. Subscriptions make this library feel similar to Publish-Subscribe. Events, though, are only for changes to properties.\n\nPropertyTable is useful when you want to expose a decent amount of state and\nlet consumers pick and choose what parts interest them.\n\nPropertyTable consumers express their interest in properties using \"patterns\". A pattern could be as simple as the property of interest or it could contain\nwildcards. This allows one to create hierarchical key-value stores, map-based\nstores, or just simple key-value stores with notifications.\n\nPropertyTable is optionally persistent to disk. Keys and values are backed by ETS.\n<!-- MODULEDOC -->","ref":"readme.html"},{"type":"extras","title":"Example - PropertyTable","doc":"While configurable, the default property style for PropertyTable is a `String` list. This enables a hierarchical key-value store. One use case that is\nroughly hierarchical is exposing network interface status to users. Imagine\na `NetworkTable` set up like the following:\n\n```sh\nNetworkTable\n├── available_interfaces\n│   └── [eth0, eth1]\n└── interface\n|   ├── eth0\n|   │   ├── config\n|   |   |   └── %{ipv4: %{method: :dhcp}}\n|   │   └── connection\n|   |       └── :internet\n|   └── eth1\n|       ├── config\n|       |   └── %{ipv4: %{method: :static}}\n|       └── connection\n|           └── :disconnected\n└── connection\n    └── :internet\n```\n\nIn this example, `NetworkTable` would be the name of the PropertyTable. The\nconnection status of \"eth1\" would be represented as `[\"interface\", \"eth1\",\n\"connection\"]` and have a value of `:disconnected`.\n\nThe library maintaining this table (the producer) creates the PropertyTable by\nadding a `child_spec` to its supervision tree:\n\n```elixir\n{PropertyTable, name: NetworkTable}\n```\n\nTo run this example from the IEx prompt, start the PropertyTable manually by\ncalling `PropertyTable.start_link/1`:\n\n```elixir\nPropertyTable.start_link(name: NetworkTable)\n```\n\nInserting properties into the table looks like:\n\n```elixir\nPropertyTable.put(NetworkTable, [\"available_interfaces\"], [\"eth0\", \"eth1\"])\nPropertyTable.put(NetworkTable, [\"connection\"], :internet)\nPropertyTable.put(NetworkTable, [\"interface\", \"eth0\", \"config\"], %{ipv4: %{method: :dhcp}})\nPropertyTable.put(NetworkTable, [\"interface\", \"eth0\", \"connection\"], :internet)\n```\n\nRead one property by running:\n\n```elixir\nPropertyTable.get(NetworkTable, [\"interface\", \"eth0\", \"config\"])\n```\n\nSince the format for properties is naturally hierarchical, you can get multiple\nby matching on a pattern that contains the start of the property that you want:\n\n```elixir\nPropertyTable.match(NetworkTable, [\"interface\"])\n```\n\nYou can subscribe to changes to receive a message after each change\nhappens. For example, to receive a message when any property starting with\n`\"interface\"` changes, run:\n\n```elixir\nPropertyTable.subscribe(table, [\"interface\"])\n```\n\nTest with:\n\n```elixir\nPropertyTable.put(NetworkTable, [\"interface\", \"eth0\", \"connection\"], :disconnected)\nflush\n```\n\nThen when a property changes value, the Erlang process that called\n`PropertyTable.subscribe/2` will receive a `%PropertyTable.Event{}` message:\n\n```elixir\n%PropertyTable.Event{\n  table: NetworkTable,\n  property: [\"interface\", \"eth0\", \"connection\"],\n  value: :disconnected\n  timestamp: 200,\n  previous_value: :internet,\n  previous_timestamp: 100\n}\n```\n\nThe timestamps in the event are from `System.monotonic_time/0`. In this example,\nyou could calculate the time that \"eth0\" was connected to the internet by\nsubtracting the timestamps.","ref":"readme.html#example"},{"type":"extras","title":"String path properties and patterns - PropertyTable","doc":"The default property format is a list of strings. Patterns are also list of\nstrings and are \"prefix\" matched. For example, the pattern `[\"a\"]` would match\nthe properties `[\"a\"]` and `[\"a\", \"b\"]` but not `[\"c\"]`. String path patterns\nalso support two positional wildcards:\n\n* `:\"$\"` - do not match paths that have additional elements\n* `:_` - match any string in that location\n\nFor example, if you want to match `[\"a\", \"b\"]` exactly, use the pattern `[\"a\",\n\"b\", :\"$\"]`. Likewise, if you don't care what's an a position in the string,\nspecify `:_` like `[:_, \"b\"]`.","ref":"readme.html#string-path-properties-and-patterns"},{"type":"extras","title":"Custom property types and patterns - PropertyTable","doc":"It's possible to replace the string path pattern matching in PropertyTable by\nproviding an implementation for the `PropertyTable.Matcher` behaviour. The\nimportant function is `matches?`:\n\n```elixir\n@callback matches?(PropertyTable.pattern(), PropertyTable.property()) :: boolean()\n```\n\nPropertyTable calls this function when deciding whether to send a property\nchange event to a subscriber and when you call `PropertyTable.match/2`.\n\nPass your module that implements the `PropertyTable.Matcher` behaviour as an\noption to PropertyTable:\n\n```elixir\n{PropertyTable, name: NetworkTable, matcher: MyCustomMatcher}\n```","ref":"readme.html#custom-property-types-and-patterns"},{"type":"extras","title":"Efficiency - PropertyTable","doc":"PropertyTable has a sweet spot in what it supports. It's not intended for very\nlarge datasets nor is it the most efficient solution for all patterns. As a\nrough guide, the use cases we had in mind with PropertyTable have in the low\n1000s of keys, a couple producers, a dozen consumers, and changes are bursty.\nOptimization choices were made with that in mind. This means:\n\n* Reads query ETS directly, but changes to properties are routed through one\n  GenServer. This reduces processing in the producer's thread context at the\n  cost of creating a potential bottleneck on multicore machines\n* Publishing events iterates over all subscriber patterns. This adds a lot of\n  flexibility to patterns. Given the design target of a dozen or so consumers,\n  it seemed that any indexing or optimization to reduce the number of\n  subscribers looked at would be slower overall than just trying each pattern.\n* Getting a specific property is very fast (one ETS looking on an indexed key),\n  but matching is slow. Matching iterates over every property. This allows for a\n  lot of flexibility in patterns. The expectation that matching is not a common\n  task, and that users will subscribe to changes over repeatedly calling match.","ref":"readme.html#efficiency"},{"type":"extras","title":"License - PropertyTable","doc":"Copyright (C) 2022 Nerves Project Authors\n\n    Licensed under the Apache License, Version 2.0 (the \"License\");\n    you may not use this file except in compliance with the License.\n    You may obtain a copy of the License at [http://www.apache.org/licenses/LICENSE-2.0](http://www.apache.org/licenses/LICENSE-2.0)\n\n    Unless required by applicable law or agreed to in writing, software\n    distributed under the License is distributed on an \"AS IS\" BASIS,\n    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n    See the License for the specific language governing permissions and\n    limitations under the License.","ref":"readme.html#license"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.2.6 - Changelog","doc":"* New feature\n  * Added `:event_transformer` option that takes a function to transform a\n    `PropertyTable.Event.t()` into a custom data structure for subscribers. This\n    lets users abstract away the use of `PropertyTable`.\n\n* Fixes\n  * The `:previous_timestamp` field in change events had been set to `nil` when\n    no information was known about the previous state of the property. It's now\n    set to the `PropertyTable`'s start time to fix the typespec violation (`nil`\n    wasn't valid) and help differentiate unknown properties due to app restarts\n    vs.  not being set.\n  * Properties set at initialization, including restored ones, all have the same\n    timestamp.","ref":"changelog.html#v0-2-6"},{"type":"extras","title":"v0.2.5 - Changelog","doc":"* Fix incorrect typespec for `PropertyTable.options()`","ref":"changelog.html#v0-2-5"},{"type":"extras","title":"v0.2.4 - Changelog","doc":"* Updates\n  * Fix unintended exceptions being raised when the filesystem updates start\n    failing when table persistence is enabled\n  * Reduce time when an unexpected VM exit could result in a corrupt persisted\n    file. The backup would be usable, but now the critical steps only involve\n    renaming or deleting files.","ref":"changelog.html#v0-2-4"},{"type":"extras","title":"v0.2.3 - Changelog","doc":"* Updates\n  * Fix compiler warnings with Elixir 1.15","ref":"changelog.html#v0-2-3"},{"type":"extras","title":"v0.2.2 - Changelog","doc":"* Updates\n  * Fixed missing `:crypto` dependency warning.","ref":"changelog.html#v0-2-2"},{"type":"extras","title":"v0.2.1 - Changelog","doc":"* New features\n  * Automatic persistence and snapshots for PropertyTables. This makes it\n    possible to use `PropertyTable` for small key/value stores like those for\n    storing runtime settings especially for Nerves devices. PropertyTable\n    protects against corruption and unexpected reboots that happen mid-write.","ref":"changelog.html#v0-2-1"},{"type":"extras","title":"v0.2.0 - Changelog","doc":"* Backwards incompatible changes\n  * `nil` no longer deletes a property from the table. In other words, it's ok to\n    for properties to be `nil` now.\n  * `PropertyTable.clear/2` and `PropertyTable.clear_all/2` were renamed to\n    `PropertyTable.delete/2` and `PropertyTable.delete_matches/2` respectively.\n  * `PropertyTable.put/3` raises if you give it an invalid property rather than\n    returning an error tuple. Since these are usually programming errors anyway,\n    this change removes the need for a return value check for Dialyzer.\n\n* New features\n  * Added `PropertyTable.put_many/2`. It's possible to add many properties to\n    the table atomically. Change notifications still only get sent for\n    properties that really changed.","ref":"changelog.html#v0-2-0"},{"type":"extras","title":"v0.1.0 - Changelog","doc":"Initial release\n\nThis code was extracted from `vintage_net`, simplified and modified to support\nmore use cases.","ref":"changelog.html#v0-1-0"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}